@@ -326,6 +326,14 @@ sub bootstrap_block {
         <name>sql_type</name>
         <default>VARCHAR(245)</default>
       </element>
+      <nested_element>
+        <name>field</name>
+        <element><name>name</name></element>
+        <element>
+          <name>sql_type</name>
+          <default>VARCHAR(245)</default>
+        </element>
+      </nested_element>
       <nested_element>
         <name>clean</name>
         <element><name>to_size</name></element>
