@@ -30,6 +30,7 @@ use File::Spec;
 # _Sd_basecalc    : Math::BaseCalc object for formatting next id
 # _Sd_max         :
 # _Sd_width       :
+# _Sd_first_digit :
 # _decl_pos       :
 
 sub MAJOR_NUMBER {
@@ -51,6 +52,7 @@ sub new {
   my $formatted_max = $self->{_Sd_basecalc}->to_base ( $self->{_Sd_max} );
   $self->{_Sd_width} = length ( $formatted_max );
   $self->{_decl_pos} = $arg{decl_pos};
+  ( $self->{_Sd_first_digit} ) = $self->{_Sd_basecalc}->digits();
   return ( $self );
 }
 
@@ -73,8 +75,10 @@ sub make_id {
   }
   return  undef  if   ! defined $next_id;
   #dbg 'next_id', $next_id;
-  $next_id = sprintf ( "%0*s", $self->{_Sd_width},
+  $next_id = sprintf ( "% *s", $self->{_Sd_width},
                        $self->{_Sd_basecalc}->to_base($next_id) );
+  my $fd = $self->{_Sd_first_digit};
+  $next_id =~ s| |$fd|g;
   return ( $next_id, # id piece
            $next_id  # location piece
          );
